import Taro, { Component } from '@tarojs/taro';
import { ComponentClass } from 'react';
import { View, Canvas, Image } from '@tarojs/components';
import BarCodeBoxBg from '../../images/barcode/tiaoxingma-1.png';
import BarCodeBoxBigBg from '../../images/barcode/tiaoxingma-2.png';
import RefreshIcon from '../../images/barcode/icon_shuaxin@2x.png';
import wxbarcode from 'wxbarcode';

import './BarCode.scss';
import { UserState } from '../../store/rootReducers/userinfo';
import { connect } from '@tarojs/redux';
import {
  fetchPayOrder,
  fetchOrderDetailAndPay,
  fetchDeductionInfo,
} from '../../api/order';
import OrderInfo from '../Order/components/OrderInfo/OrderInfo';
import OrderTitle from '../Order/components/OrderTitle/OrderTitle';
import OrderPayway from '../Order/components/OrderPayway/OrderPayway';
import {
  ConsumeOrder,
  PaymentAndActiveInfo,
  CustomerBeanAccountVo,
} from '../../api/baseClass';

type PageStateProps = {
  userinfo: UserState;
};
type PageOwnProps = {};
type PageState = {
  showPayOrder: boolean;
  showBig: boolean;
  accounts: CustomerBeanAccountVo[];
  orderInfo: ConsumeOrder;
  payInfos: {
    paymentAndActiveInfos: PaymentAndActiveInfo[];
    paymentConfId: number;
  };
  deductionInfos: {
    amiAccount: number;
    beanAccount: number;
    deductionMoney: number;
    deductionType: string;
    orderMoney: number;
    payMoney: number;
    payType: string;
    remainAccount: number;
    titlePerfix: string;
  };
};

type IProps = PageStateProps & PageOwnProps;
interface BarCode {
  props: IProps;
  state: PageState;
}

@connect(({ userinfo }) => ({
  userinfo,
}))
class BarCode extends Component {
  constructor(props: PageOwnProps) {
    super(props);
    this.state = {
      showPayOrder: false,
      showBig: false,
      accounts: [],
      orderInfo: {
        actualMoney: undefined,
        areaId: undefined,
        areaName: '',
        consumeType: '',
        createAt: '',
        customerCellphone: '',
        customerId: undefined,
        customerName: '',
        deductionBean: undefined,
        deductionMoney: undefined,
        equipmentNum: '',
        equipmentPosition: '',
        id: undefined,
        operateId: undefined,
        operateName: '',
        operationMode: '',
        orderName: '',
        orderNumber: '',
        orderState: '',
        outTradeNo: '',
        payType: '',
        payableMoney: undefined,
        serviceId: undefined,
        serviceName: '',
        thirdDiscountMoney: undefined,
        thirdTradeNumber: '',
        updateDate: '',
      },
      payInfos: {
        paymentAndActiveInfos: [],
        paymentConfId: 0,
      },
      deductionInfos: {
        amiAccount: 0,
        beanAccount: 0,
        deductionMoney: 0,
        deductionType: '',
        orderMoney: 0,
        payMoney: 0,
        payType: '',
        remainAccount: 0,
        titlePerfix: '',
      },
    };
  }

  componentWillMount() {
    const { userinfo } = this.props;
    fetchPayOrder({
      customerId: userinfo.customerId,
    })
      .then(res => {
        if (res.data.length) {
          this.fetchOrder(res.data[0].id);
          this.setState({ showPayOrder: true });
        } else {
          this.drawBarCode(false);
        }
      })
      .catch(err => {
        console.log(err);
      });
  }

  fetchOrder(id: number) {
    return fetchOrderDetailAndPay({
      id,
    })
      .then(res => {
        const { accounts, orderInfo, payInfos } = res;
        this.setState({
          accounts: accounts,
          orderInfo: orderInfo,
          payInfos: payInfos,
        });
      })
      .catch(err => {
        console.log(err);
      });
  }

  payDoneHandle() {
    const { showBig } = this.state;
    this.setState(
      {
        showPayOrder: false,
      },
      () => {
        this.drawBarCode(showBig);
      },
    );
  }
  drawBarCode(showBig: boolean) {
    const { userinfo } = this.props;
    Taro.setScreenBrightness({
      value: 1,
    });
    if (showBig) {
      wxbarcode.barcode('BarCode', userinfo.idBar, 646, 188);
    } else {
      wxbarcode.barcode('BarCode', userinfo.idBar, 700, 364);
    }
  }

  toggleBigBarCode() {
    const { showBig } = this.state;
    this.drawBarCode(showBig);
    this.setState({
      showBig: !showBig,
    });
  }

  render() {
    const { userinfo } = this.props;
    const { showPayOrder, showBig, orderInfo, payInfos, accounts } = this.state;
    return (
      <View className='BarCode'>
        {showPayOrder && (
          <View className='OrderBox'>
            <OrderTitle price={orderInfo.payableMoney} />
            <OrderInfo orderInfo={orderInfo} />
            <OrderPayway
              onPayDoneCallback={this.payDoneHandle}
              userinfo={userinfo}
              accounts={accounts}
              orderInfo={orderInfo}
              payInfos={payInfos}
            />
          </View>
        )}

        <View
          className={`BarCodeBox ${showPayOrder ? 'blur' : ''} ${
            showBig ? 'BarCodeBoxBig' : ''
          }`}>
          {showBig ? (
            <Image className='bg' src={BarCodeBoxBigBg} />
          ) : (
            <Image className='bg' src={BarCodeBoxBg} />
          )}
          {!showBig && (
            <Image className='BarCodeBox-refresh' src={RefreshIcon} />
          )}
          <View className={`BarCodeImg ${showBig ? 'BarCodeImgBig' : ''}`}>
            {showPayOrder ? null : (
              <Canvas className='BarCodeCav' canvasId='BarCode' />
            )}
          </View>
          <View className='toggleBtn' onClick={this.toggleBigBarCode}>
            {showBig ? '缩小' : '放大'}
          </View>
        </View>
      </View>
    );
  }
}

export default BarCode as ComponentClass<PageOwnProps, PageState>;
